import os
from pathlib import Path
from typing import Any, Dict

from django.conf.global_settings import LANGUAGES as DJANGO_LANGUAGES

###################
# Django Settings #
###################
BASE_DIR = Path(__file__).resolve().parent
SECRET_KEY = os.getenv('DJANGO_SECRET_KEY', 'not-secret-at-all')
DEBUG = bool(int(os.getenv('DEBUG', 1)))
TEST = os.getenv('FAIL_INVALID_TEMPLATE_VARS')

PREFIX = '' if os.getenv('STANDALONE') else ''

ALLOWED_HOSTS = ['*']
INSTALLED_APPS = [
    # Keep this above 'django.contrib.admin'
    'richmin',
    # Django apps
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Our apps
    '{}library.books.apps.BooksConfig'.format(PREFIX),
    '{}library.loans.apps.LoansConfig'.format(PREFIX),
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.admindocs.middleware.XViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = '{}library.urls'.format(PREFIX)

WSGI_APPLICATION = '{}library.wsgi.application'.format(PREFIX)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler'
        }
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': 'INFO'
        }
    },
}

TEMPLATES = [{
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'DIRS': [],
    'APP_DIRS': True,
    'OPTIONS': {
        'context_processors': [
            'django.template.context_processors.debug',
            'django.template.context_processors.request',
            'django.contrib.auth.context_processors.auth',
            'django.contrib.messages.context_processors.messages',
        ],
    },
}]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator'
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator'
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator'
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator'
    },
]

LANGUAGE_CODE = 'en'
TIME_ZONE = 'Europe/London'
USE_I18N = True
LOCALE_PATHS = (os.path.join(BASE_DIR, 'locale'),)
USE_L10N = True
USE_TZ = True

# English default
LANGUAGES = DJANGO_LANGUAGES

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')

MEDIA_URL = '/media/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

if DEBUG and not TEST:
    os.environ.setdefault('WERKZEUG_DEBUG_PIN', 'off')
    INSTALLED_APPS.extend(['debug_toolbar', 'django_extensions'])
    MIDDLEWARE.insert(0, 'debug_toolbar.middleware.DebugToolbarMiddleware')
    DEBUG_TOOLBAR_CONFIG = {'SHOW_TOOLBAR_CALLBACK': lambda _: False}

if not DEBUG and not TEST:
    MIDDLEWARE.insert(1, 'whitenoise.middleware.WhiteNoiseMiddleware')
    STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'

########################
# Third party settings #
########################
RICHMIN_SETTINGS: Dict[str, Any] = {
    # title of the window (Will default to current_admin_site.site_title if absent or None)
    'site_title': 'Library Admin',
    # Title on the brand, and login screen (19 chars max) (defaults to current_admin_site.site_header if absent or None)
    'site_header': 'Library',
    # Logo to use for your site, must be present in static files, used for brand on top left
    'site_logo': 'books/img/logo.png',
    # Relative path to logo for your site, used for login logo (must be present in static files. Defaults to site_logo)
    'login_logo': 'books/img/logo-login.png',
    # Logo to use for login form in dark themes (must be present in static files. Defaults to login_logo)
    'login_logo_dark': 'books/img/logo-login-dark-mode.png',
    # CSS classes that are applied to the logo
    'site_logo_classes': None,
    # Relative path to a favicon for your site, will default to site_logo if absent (ideally 32x32 px)
    'site_icon': 'books/img/icon.png',
    # Welcome text on the login screen
    'welcome_sign': 'Welcome to the library',
    # Copyright on the footer
    'copyright': 'Richmin Library',
    # List of model admins to search from the search bar, search bar omitted if excluded
    # If you want to use a single search field you dont need to use a list, you can use a simple string
    'search_model': ['auth.User', 'auth.Group'],
    # Field name on user model that contains avatar ImageField/URLField/Charfield or a callable that receives the user
    'user_avatar': None,
    ############
    # Top Menu #
    ############
    # Links to put along the top menu
    'topmenu_links': [
        # Url that gets reversed (Permissions can be added)
        {
            'name': 'Home',
            'url': 'admin:index',
            'permissions': ['auth.view_user']
        },
        # external url that opens in a new window (Permissions can be added)
        {
            'name': 'Support',
            'url': 'https://github.com/mymy47/django-richmin',
            'new_window': True
        },
        # model admin to link to (Permissions checked against model)
        {
            'model': 'auth.User'
        },
        # App with dropdown menu to all its models pages (Permissions checked against models)
        {
            'app': 'books'
        },
        {
            'app': 'loans'
        },
    ],
    #############
    # User Menu #
    #############
    # Additional links to include in the user menu on the top right ('app' url type is not allowed)
    'usermenu_links': [
        {
            'name': 'Support',
            'url': 'https://github.com/mymy47/django-richmin',
            'new_window': True
        },
        {
            'model': 'auth.user'
        },
    ],
    #############
    # Side Menu #
    #############
    # Whether to display the side menu
    'show_sidebar': True,
    # Whether to aut expand the menu
    'navigation_expanded': True,
    # Hide these apps when generating side menu e.g (auth)
    'hide_apps': [],
    # Hide these models when generating side menu (e.g auth.user)
    'hide_models': [],
    # List of apps to base side menu (app or model) ordering off of
    'order_with_respect_to': ['Make Messages', 'auth', 'books', 'books.author', 'books.book', 'loans'],
    # Custom links to append to app groups, keyed on app name
    'custom_links': {
        'loans': [
            {
                'name': 'Make Messages',
                'url': 'make_messages',
                'icon': 'fas fa-comments',
                'permissions': ['loans.view_loan'],
            },
            {
                'name': 'Custom View',
                'url': 'admin:custom_view',
                'icon': 'fas fa-box-open'
            },
        ]
    },
    # Custom icons for side menu apps/models See the link below
    # https://fontawesome.com/icons?d=gallery&m=free&v=5.0.0,5.0.1,5.0.10,5.0.11,5.0.12,5.0.13,5.0.2,5.0.3,5.0.4,5.0.5,5.0.6,5.0.7,5.0.8,5.0.9,5.1.0,
    # 5.1.1,5.2.0,5.3.0,5.3.1,5.4.0,5.4.1,5.4.2,5.13.0,5.12.0,5.11.2,5.11.1,5.10.0,5.9.0,5.8.2,5.8.1,5.7.2,5.7.1,5.7.0,5.6.3,5.5.0,5.4.2
    # for the full list of 5.13.0 free icon classes
    'icons': {
        'auth': 'fas fa-users-cog',
        'auth.user': 'fas fa-user',
        'auth.Group': 'fas fa-users',
        'admin.LogEntry': 'fas fa-file',
        'books.Author': 'fas fa-user',
        'books.Book': 'fas fa-book',
        'books.Genre': 'fas fa-photo-video',
        'loans.BookLoan': 'fas fa-book-open',
        'loans.Library': 'fas fa-book-reader',
    },
    # Icons that are used when one is not manually specified
    'default_icon_parents': 'fas fa-chevron-circle-right',
    'default_icon_children': 'fas fa-circle',
    #################
    # Related Modal #
    #################
    # Activate Bootstrap modal
    'related_modal_active': False,
    #############
    # UI Tweaks #
    #############
    # Relative paths to custom CSS/JS scripts (must be present in static files)
    'custom_css': None,
    'custom_js': None,
    # Whether to show the UI customizer on the sidebar
    'show_ui_builder': True,
    ###############
    # Change view #
    ###############
    # Render out the change view as a single form, or in tabs, current options are
    # - single
    # - horizontal_tabs (default)
    # - vertical_tabs
    # - collapsible
    # - carousel
    'changeform_format': 'horizontal_tabs',
    # override change forms on a per modeladmin basis
    'changeform_format_overrides': {
        'auth.user': 'collapsible',
        'auth.group': 'vertical_tabs'
    },
    # Add a language dropdown into the admin
    'language_chooser': True,
}

if not DEBUG and not TEST:
    RICHMIN_SETTINGS['welcome_sign'] = 'Username: test@test.com, Password: test (Data resets nightly)'

RICHMIN_UI_TWEAKS = {
    'navbar_small_text': False,
    'footer_small_text': False,
    'body_small_text': False,
    'brand_small_text': False,
    'brand_colour': False,
    'accent': 'accent-primary',
    'navbar': 'navbar-white navbar-light',
    'no_navbar_border': False,
    'navbar_fixed': False,
    'layout_boxed': False,
    'footer_fixed': False,
    'sidebar_fixed': False,
    'sidebar': 'sidebar-dark-primary',
    'sidebar_nav_small_text': False,
    'sidebar_disable_expand': False,
    'sidebar_nav_child_indent': False,
    'sidebar_nav_compact_style': False,
    'sidebar_nav_legacy_style': False,
    'sidebar_nav_flat_style': False,
    'theme': 'default',
    'dark_mode_theme': 'darkly',
    'button_classes': {
        'primary': 'btn-outline-primary',
        'secondary': 'btn-outline-secondary',
        'info': 'btn-outline-info',
        'warning': 'btn-outline-warning',
        'danger': 'btn-outline-danger',
        'success': 'btn-outline-success',
    },
}

################
# App settings #
################
